%define ALD_Ver		1.2.4

#USE gcc-4.9.4

%define OrgName uClibc

License:      LGPL
Summary:      GNU The uClibc static C libraries
Group:        Development/uC-Libraries
Version:      1.0.25
Release:      0

Source: %{OrgName}-ng-%{version}.tar.xz
Source1: %{OrgName}-ng-%{version}-patches.tgz

%define DestDir		/Embedded/uC-%{version}

# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mhard-float
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1

# -----------------------------------------------------

%if %{BUILD_LOCAL_CROSS}
Name:         %{OrgName}-%{TO_ARCH}
%else
Name:         %{OrgName}
%endif

%define _prefix /usr/uC/%{version}/static
%define _lib lib

#------------------ CROSS DIR --------------------
%if %{_CROSS}

%if %{_os} == "mingw"
ERROR: target=%{_os}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif
%define target_  %{target}-

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/bin

%if %{BUILD_LOCAL_CROSS}
%define _prefix		%{csysdir}/usr/uC/%{version}/static
%define CrossDestDir	/Cross
%else
%define CrossDestDir %{nil}
%endif

%define __os_install_post \
    %{csysbindir}/rpm/brp-strip \
    %{csysbindir}/rpm/brp-strip-shared \
    %{csysbindir}/rpm/brp-strip-static-archive \
    %{csysbindir}/rpm/brp-strip-comment-note \
    %{_rpmconfigdir}/brp-compress \
%{nil}

%define _srcrpmdir %{_topdir}/SRPMS/tmp

%else  #_CROSS

%define CrossDestDir %{nil}
%define _srcrpmdir %{_topdir}/SRPMS/%{ALD_Ver}

%define target_  %{nil}

%endif # !_CROSS

%define docdir %{_defaultdocdir}/%{name}

# ------------------- description ------------------------------------
%description
uClibc - a Small C Library for Linux

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n %{OrgName}-ng-%{version}

tar -xzf %SOURCE1
tar -xzf patches/uClibc-gcc.cross.tar.gz
patch -b -z .orig -p0<patches/uClibc-gcc.patch
patch -b -z .orig -p0<patches/Enable_CTOR_DTOR_support.patch

%if %{_arch} == "i386"
cp patches/i386.config .config
%endif

%if %{_arch} == "x86_64"
cp patches/x86_64.config .config
%endif

%if (%{_target_cpu} == "armv6l") || (%{_target_cpu} == "armv7l")
patch -b -z .orig -p0<patches/nptl-static.patch
cp patches/arm.config .config
%endif

%if %{_CROSS}
sed -i s:"CROSS_COMPILER_PREFIX=\"\"":"CROSS_COMPILER_PREFIX=\"%{target}-\"": .config
sed -i s:"/usr/include":"%{csysdir}/usr/include": .config
%endif
%if %{BUILD_LOCAL_CROSS}
sed -i s:"/usr/uC":"%{csysdir}/usr/uC": .config
%endif

# ------------------- Build ------------------------------------------
%Build
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif


%if %{BUILD_LOCAL_CROSS}
CC_HOST="gcc"
GCC_BIN_HOST="%{csysbindir}/%{target}-gcc"
LD_BIN_HOST="%{csysbindir}/%{target}-ld"
%else  # BUILD_LOCAL_CROSS
%if %{_CROSS}
CC_HOST="%{target}-gcc"
%else
CC_HOST="gcc"
%endif
GCC_BIN_HOST="/usr/bin/gcc"
LD_BIN_HOST="/usr/bin/ld"
%endif  # ! BUILD_LOCAL_CROSS


GCC_VER_STR=`%{target_}gcc -dumpversion`
OPT47=`case $GCC_VER_STR in 4.[789].*) echo "-fno-asynchronous-unwind-tables" ;; esac`

%if %{_arch} == "i386"
CFLAGS_HOST="-Os -march=i386 $OPT47 -fomit-frame-pointer"
%else
CFLAGS_HOST="-Os $OPT47 -fomit-frame-pointer"
%endif

export CC_HOST GCC_BIN_HOST CFLAGS_HOST LD_BIN_HOST

make VERBOSE=1 &>FMake_log.txt

# ------------------- Install ----------------------------------------
%Install
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
export PATH
%else
export PATH=$PATH:/opt/perl/bin
%endif

%if %{BUILD_LOCAL_CROSS}
LD_BIN_HOST="%{csysbindir}/%{target}-ld"
%else
LD_BIN_HOST="/usr/bin/ld"
%endif

export LD_BIN_HOST


rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}

PREFIX="$RPM_BUILD_ROOT"
export PREFIX
%makeinstall &>FInstall_log.txt

cp -a README $RPM_BUILD_ROOT%{docdir}/
ln -s /usr/share/doc/lgpl-2.1.txt $RPM_BUILD_ROOT%{docdir}/COPYING.LIB

pushd $RPM_BUILD_ROOT%{_bindir}
%if %{BUILD_LOCAL_CROSS}
rm addr2line ar as cc cpp gcc ld nm objcopy objdump ranlib size strings strip

for sf in addr2line ar as cpp nm objcopy objdump ranlib size strings strip; do
  ln -s %{csysdir}/bin/%{target}-$sf
  rm %{_arch}-uclibc-$sf
  ln -s %{csysdir}/bin/%{target}-$sf %{_arch}-uclibc-$sf
done  
ln -s %{_arch}-uclibc-gcc %{target}-gcc
ln -s %{_arch}-uclibc-gcc %{target}-cc
ln -s %{_arch}-uclibc-ld %{target}-ld
%else
rm cc gcc ld
ln -s %{_arch}-uclibc-gcc gcc
ln -s %{_arch}-uclibc-gcc cc
ln -s %{_arch}-uclibc-ld ld
%endif
popd


pushd $RPM_BUILD_ROOT%{_includedir}
%if %{BUILD_LOCAL_CROSS}
ln -s %{csysdir}/usr/include/asm
ln -s %{csysdir}/usr/include/asm-generic
ln -s %{csysdir}/usr/include/linux
%else
ln -s /usr/include/asm
ln -s /usr/include/asm-generic
ln -s /usr/include/linux
%endif
popd

# ------------------- Clean ------------------------------------------
%Clean

%if %{BUILD_LOCAL_CROSS}
  DE=%{version}-%{release}.%{_build_cpu}.rpm
  DD=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

%else  #%{BUILD_LOCAL_CROSS}
  DE=%{version}-%{release}.%{TO_ARCH}.rpm
  DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{CrossDestDir}%{DestDir}
%endif #!%{BUILD_LOCAL_CROSS}
 
pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
  TMP_TO=$DD/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}
  mkdir -p $tmp
%if %{BUILD_LOCAL_CROSS}
  cp -r cross $tmp/
%else
  cp -r usr $tmp/
%endif
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
popd
%endif  #BUILD_LOCAL_CROSS

%post
%if %{BUILD_LOCAL_CROSS}
if test ! -L %{csysdir}/usr/uC/static; then
pushd %{csysdir}/usr/uC >/dev/null
ln -s %{version}/static static
popd >/dev/null
fi
%else
if test ! -L /usr/uC/static; then
pushd /usr/uC >/dev/null
ln -s %{version}/static static
popd >/dev/null
fi
%endif
# ------------------- Files ------------------------------------------
%Files
%if %{BUILD_LOCAL_CROSS}
%{csysdir}
%else
/usr
%endif
